# coding=utf-8
import contextlib
import subprocess as sp
import threading
import time
from argparse import ArgumentDefaultsHelpFormatter, ArgumentParser

import depthai as dai

# Parse command line arguments
parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
parser.add_argument(
    "-rip", "--rtsp_host", default="0.0.0.0", type=str, help="Host of the RTSP Server"
)
parser.add_argument(
    "-rp", "--rtsp_port", default=8554, type=int, help="Port of the RTSP Server"
)
parser.add_argument(
    "-wt",
    "--width",
    default=1920,
    type=int,
    help="Width of the video/preview size. In multiple of 32",
)
parser.add_argument(
    "-ht",
    "--height",
    default=1080,
    type=int,
    help="Height of the video/preview size. In multiple of 8",
)
parser.add_argument(
    "-qa", "--quality", default=100, type=int, help="Video quality, from 1 to 100"
)
parser.add_argument(
    "-sm",
    "--scale_mode",
    default=True,
    type=bool,
    help="Scale or crop the video output. Default is scale. Set to false to switch to crop mode",
)
args = vars(parser.parse_args())

RTSP_PORT = args["rtsp_port"]
RTSP_HOST = args["rtsp_host"]
WIDTH = args["width"]
HEIGHT = args["height"]
QUALITY = args["quality"]
SCALE_ON = args["scale_mode"]


def getPipeline():
    pipeline = dai.Pipeline()

    FPS = 30
    colorCam = pipeline.create(dai.node.ColorCamera)
    colorCam.setResolution(dai.ColorCameraProperties.SensorResolution.THE_1080_P)
    colorCam.setInterleaved(False)
    colorCam.setColorOrder(dai.ColorCameraProperties.ColorOrder.BGR)
    colorCam.setFps(FPS)
    colorCam.setPreviewSize(WIDTH, HEIGHT)
    colorCam.setVideoSize(WIDTH, HEIGHT)
    if SCALE_ON is True:
        colorCam.setIspScale(WIDTH, 1920)

    videnc = pipeline.create(dai.node.VideoEncoder)
    videnc.setDefaultProfilePreset(FPS, dai.VideoEncoderProperties.Profile.H264_MAIN)
    videnc.setKeyframeFrequency(FPS * 4)
    videnc.setQuality(QUALITY)

    colorCam.video.link(videnc.input)

    veOut = pipeline.create(dai.node.XLinkOut)
    veOut.setStreamName("encoded")
    videnc.bitstream.link(veOut.input)

    return pipeline


run = True


def worker(dev_info, stack, i):
    global run
    if dev_info.protocol != dai.XLinkProtocol.X_LINK_USB_VSC:
        print(
            "Running RTSP stream may be unstable due to connection... (protocol: {})".format(
                dev_info.protocol
            )
        )

    command = [
        "ffmpeg",
        # "-re",
        "-fflags",
        "+genpts",
        "-probesize",
        "100M",
        "-i",
        "-",
        "-framerate",
        "30",
        "-vcodec",
        "copy",
        "-v",
        "error",
        "-f",
        "rtsp",
        f"rtsp://{RTSP_HOST}:{RTSP_PORT}/preview/{i}",
    ]

    try:
        proc = sp.Popen(command, stdin=sp.PIPE)  # Start the ffmpeg process
    except:
        exit("Error: cannot run ffmpeg!\nTry running: sudo apt install ffmpeg")

    openvino_version = dai.OpenVINO.Version.VERSION_2021_4
    device: dai.Device = stack.enter_context(
        dai.Device(openvino_version, dev_info, False)
    )

    # Note: currently on POE, DeviceInfo.getMxId() and Device.getMxId() are different!
    print("=== Connected to " + dev_info.getMxId())
    mxid = device.getMxId()
    cameras = device.getConnectedCameras()
    usb_speed = device.getUsbSpeed()
    print("   >>> MXID:", mxid)
    print("   >>> Cameras:", *[c.name for c in cameras])
    print("   >>> USB speed:", usb_speed.name)

    device.startPipeline(getPipeline())
    encoded = device.getOutputQueue("encoded", maxSize=40, blocking=True)

    print(f"Play: ffplay {command[-1]}")

    while run:
        # Blocking call, will wait until new data has arrived

        data = encoded.get().getData()
        proc.stdin.write(data)

    proc.stdin.close()

    print("Closing thread")


if __name__ == "__main__":
    try:
        device_infos = dai.Device.getAllAvailableDevices()
        print(f"Found {len(device_infos)} devices")
        print("Press Ctrl+C to stop streaming ...")
        with contextlib.ExitStack() as stack:
            threads = []
            for i, dev in enumerate(device_infos):
                time.sleep(1)  # Currently required due to XLink race issues
                thread = threading.Thread(target=worker, args=(dev, stack, i))
                thread.start()
                threads.append(thread)

            for t in threads:
                t.join()  # Wait for all threads to finish

    except KeyboardInterrupt:
        run = False
